@@ -5,9 +5,9 @@ fn create_svn_bindings(
     out_path: &std::path::Path,
     include_paths: &[&std::path::Path],
 ) {
-    // Generate bindings using bindgen
-    let svn_bindings = bindgen::Builder::default()
-        .header(svn_path.join("svn_client.h").to_str().unwrap())
+    let client_feature_enabled = std::env::var("CARGO_FEATURE_CLIENT").is_ok();
+
+    let mut builder = bindgen::Builder::default()
         .header(svn_path.join("svn_dirent_uri.h").to_str().unwrap())
         .header(svn_path.join("svn_version.h").to_str().unwrap())
         .header(svn_path.join("svn_error.h").to_str().unwrap())
@@ -25,9 +25,14 @@ fn create_svn_bindings(
             include_paths
                 .iter()
                 .map(|path| format!("-I{}", path.display())),
-        )
-        .generate()
-        .expect("Failed to generate bindings");
+        );
+
+    if client_feature_enabled {
+        builder = builder.header(svn_path.join("svn_client.h").to_str().unwrap());
+    }
+
+    // Generate bindings using bindgen
+    let svn_bindings = builder.generate().expect("Failed to generate bindings");
 
     svn_bindings
         .write_to_file(out_path.join("subversion.rs"))
@@ -37,13 +42,13 @@ fn create_svn_bindings(
 fn main() {
     let deps = system_deps::Config::new().probe().unwrap();
 
-    let svn = deps.get_by_name("libsvn_client").unwrap();
+    let svn = deps.get_by_name("libsvn_subr").unwrap();
 
     let svn_path = svn
         .include_paths
         .iter()
-        .find(|x| x.join("svn_client.h").exists())
-        .expect("Failed to find svn_client.h");
+        .find(|x| x.join("svn_config.h").exists())
+        .expect("Failed to find svn_config.h");
 
     let out_path = std::path::PathBuf::from(std::env::var("OUT_DIR").unwrap());
     create_svn_bindings(
