@@ -18,24 +18,6 @@ pub fn version() -> Version {
     unsafe { Version(svn_client_version()) }
 }
 
-extern "C" fn wrap_commit_callback2(
-    commit_info: *const crate::generated::svn_commit_info_t,
-    baton: *mut std::ffi::c_void,
-    pool: *mut apr::apr_pool_t,
-) -> *mut crate::generated::svn_error_t {
-    unsafe {
-        let callback = baton as *mut &mut dyn FnMut(&crate::CommitInfo) -> Result<(), Error>;
-        let mut callback = Box::from_raw(callback);
-        match callback(&crate::CommitInfo(PooledPtr::in_pool(
-            std::rc::Rc::new(Pool::from_raw(pool)),
-            commit_info as *mut crate::generated::svn_commit_info_t,
-        ))) {
-            Ok(()) => std::ptr::null_mut(),
-            Err(mut err) => err.as_mut_ptr(),
-        }
-    }
-}
-
 extern "C" fn wrap_filter_callback(
     baton: *mut std::ffi::c_void,
     filtered: *mut crate::generated::svn_boolean_t,
@@ -80,27 +62,6 @@ extern "C" fn wrap_status_func(
     }
 }
 
-extern "C" fn wrap_log_entry_receiver(
-    baton: *mut std::ffi::c_void,
-    log_entry: *mut crate::generated::svn_log_entry_t,
-    pool: *mut apr::apr_pool_t,
-) -> *mut crate::generated::svn_error_t {
-    unsafe {
-        let callback = baton as *mut &mut dyn FnMut(&LogEntry) -> Result<(), Error>;
-        let mut callback = Box::from_raw(callback);
-        let pool = apr::pool::Pool::from_raw(pool);
-        let ret = callback(&LogEntry(apr::pool::PooledPtr::in_pool(
-            std::rc::Rc::new(pool),
-            log_entry,
-        )));
-        if let Err(mut err) = ret {
-            err.as_mut_ptr()
-        } else {
-            std::ptr::null_mut()
-        }
-    }
-}
-
 extern "C" fn wrap_proplist_receiver2(
     baton: *mut std::ffi::c_void,
     path: *const i8,
@@ -164,8 +125,8 @@ impl Info {
         }
     }
 
-    pub fn revision(&self) -> Revision {
-        unsafe { Revision::from((*self.0).rev) }
+    pub fn revision(&self) -> Revnum {
+        unsafe { Revnum::from_raw((*self.0).rev).unwrap() }
     }
 
     pub fn kind(&self) -> crate::generated::svn_node_kind_t {
@@ -187,7 +148,7 @@ impl Info {
     }
 
     pub fn last_changed_rev(&self) -> Revnum {
-        unsafe { (*self.0).last_changed_rev }
+        Revnum::from_raw(unsafe { (*self.0).last_changed_rev }).unwrap()
     }
 
     pub fn last_changed_date(&self) -> apr::time::Time {
@@ -224,11 +185,153 @@ extern "C" fn wrap_info_receiver2(
     }
 }
 
+/// Options for cat
+#[derive(Debug, Clone, Copy, Default)]
+pub struct CatOptions {
+    revision: Revision,
+    peg_revision: Revision,
+    expand_keywords: bool,
+}
+
+/// Options for cleanup
+#[derive(Debug, Clone, Copy, Default)]
+pub struct CleanupOptions {
+    break_locks: bool,
+    fix_recorded_timestamps: bool,
+    clear_dav_cache: bool,
+    vacuum_pristines: bool,
+    include_externals: bool,
+}
+
+/// Options for proplist
+#[derive(Debug, Clone, Copy, Default)]
+pub struct ProplistOptions<'a> {
+    peg_revision: Revision,
+    revision: Revision,
+    depth: Depth,
+    changelists: Option<&'a [&'a str]>,
+    get_target_inherited_props: bool,
+}
+
+/// Options for export
+#[derive(Debug, Clone, Copy, Default)]
+pub struct ExportOptions {
+    peg_revision: Revision,
+    revision: Revision,
+    overwrite: bool,
+    ignore_externals: bool,
+    ignore_keywords: bool,
+    depth: Depth,
+    native_eol: crate::NativeEOL,
+}
+
+/// Options for vacuum
+#[derive(Debug, Clone, Copy, Default)]
+pub struct VacuumOptions {
+    remove_unversioned_items: bool,
+    remove_ignored_items: bool,
+    fix_recorded_timestamps: bool,
+    vacuum_pristines: bool,
+    include_externals: bool,
+}
+
+/// Options for a checkout
+#[derive(Debug, Clone, Copy, Default)]
+pub struct CheckoutOptions {
+    peg_revision: Revision,
+    revision: Revision,
+    depth: Depth,
+    ignore_externals: bool,
+    allow_unver_obstructions: bool,
+}
+
+/// Options for an update
+#[derive(Debug, Clone, Copy, Default)]
+pub struct UpdateOptions {
+    depth: Depth,
+    depth_is_sticky: bool,
+    ignore_externals: bool,
+    allow_unver_obstructions: bool,
+    adds_as_modifications: bool,
+    make_parents: bool,
+}
+
+/// Options for a switch
+pub struct SwitchOptions {
+    peg_revision: Revision,
+    revision: Revision,
+    depth: Depth,
+    depth_is_sticky: bool,
+    ignore_externals: bool,
+    allow_unver_obstructions: bool,
+    make_parents: bool,
+}
+
+/// Options for add
+pub struct AddOptions {
+    depth: Depth,
+    force: bool,
+    no_ignore: bool,
+    no_autoprops: bool,
+    add_parents: bool,
+}
+
+/// Options for delete
+pub struct DeleteOptions<'a> {
+    force: bool,
+    keep_local: bool,
+    commit_callback: &'a dyn FnMut(&crate::CommitInfo) -> Result<(), Error>,
+}
+
+/// Options for commit
+pub struct CommitOptions<'a> {
+    depth: Depth,
+    keep_locks: bool,
+    keep_changelists: bool,
+    commit_as_operations: bool,
+    include_file_externals: bool,
+    include_dir_externals: bool,
+    changelists: Option<&'a [&'a str]>,
+}
+
+/// Options for status
+pub struct StatusOptions<'a> {
+    revision: Revision,
+    depth: Depth,
+    get_all: bool,
+    check_out_of_date: bool,
+    check_working_copy: bool,
+    no_ignore: bool,
+    ignore_externals: bool,
+    depth_as_sticky: bool,
+    changelists: Option<&'a [&'a str]>,
+}
+
 /// A client context.
 ///
 /// This is the main entry point for the client library. It holds client specific configuration and
 /// callbacks
 pub struct Context(apr::pool::PooledPtr<svn_client_ctx_t>);
+unsafe impl Send for Context {}
+
+impl Clone for Context {
+    fn clone(&self) -> Self {
+        let new = Context::new().unwrap();
+
+        // TODO: Copy auth_baton
+        // TODO: Copy notify func
+        // TODO: copy log_msg_func
+        // TODO: copy config
+        // TODO: copy cancel_func
+        // TODO: copy progress_func
+        // TODO: copy wc_ctx
+        // TODO: copy conflict_func
+        // TODO: copy mimetypes map
+        // TODO: copy check_tunnel_func
+        // TODO: copy open_tunnel_func
+        new
+    }
+}
 
 impl Context {
     pub fn new() -> Result<Self, Error> {
@@ -243,14 +346,10 @@ impl Context {
         })?))
     }
 
-    pub fn as_mut_ptr(&mut self) -> *mut svn_client_ctx_t {
+    pub(crate) unsafe fn as_mut_ptr(&mut self) -> *mut svn_client_ctx_t {
         self.0.as_mut_ptr()
     }
 
-    pub fn as_ptr(&self) -> *const svn_client_ctx_t {
-        self.0.as_ptr()
-    }
-
     pub fn set_auth<'a, 'b>(&'a mut self, auth_baton: &'b mut crate::auth::AuthBaton)
     where
         'b: 'a,
@@ -263,15 +362,10 @@ impl Context {
         &mut self,
         url: impl AsCanonicalUri<'a>,
         path: impl AsCanonicalDirent<'a>,
-        peg_revision: Revision,
-        revision: Revision,
-        depth: Depth,
-        ignore_externals: bool,
-        allow_unver_obstructions: bool,
+        options: &CheckoutOptions,
     ) -> Result<Revnum, Error> {
-        // call svn_client_checkout2
-        let peg_revision = peg_revision.into();
-        let revision = revision.into();
+        let peg_revision = options.peg_revision.into();
+        let revision = options.revision.into();
         let mut pool = Pool::default();
         unsafe {
             let mut revnum = 0;
@@ -283,14 +377,14 @@ impl Context {
                 path.as_ptr(),
                 &peg_revision,
                 &revision,
-                depth.into(),
-                ignore_externals.into(),
-                allow_unver_obstructions.into(),
+                options.depth.into(),
+                options.ignore_externals.into(),
+                options.allow_unver_obstructions.into(),
                 &mut *self.0,
                 pool.as_mut_ptr(),
             );
             Error::from_raw(err)?;
-            Ok(revnum)
+            Ok(Revnum::from_raw(revnum).unwrap())
         }
     }
 
@@ -298,12 +392,7 @@ impl Context {
         &mut self,
         paths: &[&str],
         revision: Revision,
-        depth: Depth,
-        depth_is_sticky: bool,
-        ignore_externals: bool,
-        allow_unver_obstructions: bool,
-        adds_as_modifications: bool,
-        make_parents: bool,
+        options: &UpdateOptions,
     ) -> Result<Vec<Revnum>, Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         let mut result_revs = std::ptr::null_mut();
@@ -318,12 +407,12 @@ impl Context {
                 &mut result_revs,
                 ps.as_ptr(),
                 &revision.into(),
-                depth.into(),
-                depth_is_sticky.into(),
-                ignore_externals.into(),
-                allow_unver_obstructions.into(),
-                adds_as_modifications.into(),
-                make_parents.into(),
+                options.depth.into(),
+                options.depth_is_sticky.into(),
+                options.ignore_externals.into(),
+                options.allow_unver_obstructions.into(),
+                options.adds_as_modifications.into(),
+                options.make_parents.into(),
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
             );
@@ -338,13 +427,7 @@ impl Context {
         &mut self,
         path: impl AsCanonicalDirent<'a>,
         url: impl AsCanonicalUri<'a>,
-        peg_revision: Revision,
-        revision: Revision,
-        depth: Depth,
-        depth_is_sticky: bool,
-        ignore_externals: bool,
-        allow_unver_obstructions: bool,
-        make_parents: bool,
+        options: &SwitchOptions,
     ) -> Result<Revnum, Error> {
         let mut pool = Pool::default();
         let mut result_rev = 0;
@@ -355,40 +438,36 @@ impl Context {
                 &mut result_rev,
                 path.as_ptr(),
                 url.as_ptr(),
-                &peg_revision.into(),
-                &revision.into(),
-                depth.into(),
-                depth_is_sticky.into(),
-                ignore_externals.into(),
-                allow_unver_obstructions.into(),
-                make_parents.into(),
+                &options.peg_revision.into(),
+                &options.revision.into(),
+                options.depth.into(),
+                options.depth_is_sticky.into(),
+                options.ignore_externals.into(),
+                options.allow_unver_obstructions.into(),
+                options.make_parents.into(),
                 &mut *self.0,
                 pool.as_mut_ptr(),
             );
             Error::from_raw(err)?;
-            Ok(result_rev)
+            Ok(Revnum::from_raw(result_rev).unwrap())
         }
     }
 
     pub fn add<'a>(
         &mut self,
         path: impl AsCanonicalDirent<'a>,
-        depth: Depth,
-        force: bool,
-        no_ignore: bool,
-        no_autoprops: bool,
-        add_parents: bool,
+        options: &AddOptions,
     ) -> Result<(), Error> {
         let mut pool = Pool::default();
         let path = path.as_canonical_dirent(&mut pool);
         unsafe {
             let err = svn_client_add5(
                 path.as_ptr(),
-                depth.into(),
-                force.into(),
-                no_ignore.into(),
-                no_autoprops.into(),
-                add_parents.into(),
+                options.depth.into(),
+                options.force.into(),
+                options.no_ignore.into(),
+                options.no_autoprops.into(),
+                options.add_parents.into(),
                 &mut *self.0,
                 pool.as_mut_ptr(),
             );
@@ -420,7 +499,7 @@ impl Context {
                 ps.as_ptr(),
                 make_parents.into(),
                 rps.as_ptr(),
-                Some(wrap_commit_callback2),
+                Some(crate::wrap_commit_callback2),
                 commit_callback as *mut std::ffi::c_void,
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -433,10 +512,8 @@ impl Context {
     pub fn delete(
         &mut self,
         paths: &[&str],
-        force: bool,
-        keep_local: bool,
         revprop_table: std::collections::HashMap<&str, &str>,
-        commit_callback: &dyn FnMut(&crate::CommitInfo) -> Result<(), Error>,
+        options: &DeleteOptions,
     ) -> Result<(), Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         unsafe {
@@ -449,13 +526,13 @@ impl Context {
                 let path = std::ffi::CString::new(*path).unwrap();
                 ps.push(path.as_ptr() as *mut std::ffi::c_void);
             }
-            let commit_callback = Box::into_raw(Box::new(commit_callback));
+            let commit_callback = Box::into_raw(Box::new(options.commit_callback));
             let err = svn_client_delete4(
                 ps.as_ptr(),
-                force.into(),
-                keep_local.into(),
+                options.force.into(),
+                options.keep_local.into(),
                 rps.as_ptr(),
-                Some(wrap_commit_callback2),
+                Some(crate::wrap_commit_callback2),
                 commit_callback as *mut std::ffi::c_void,
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -468,20 +545,16 @@ impl Context {
     pub fn proplist(
         &mut self,
         target: &str,
-        peg_revision: Revision,
-        revision: Revision,
-        depth: Depth,
-        changelists: Option<&[&str]>,
-        get_target_inherited_props: bool,
+        options: &ProplistOptions,
         receiver: &mut dyn FnMut(
             &str,
             &std::collections::HashMap<String, Vec<u8>>,
             Option<&[crate::InheritedItem]>,
         ) -> Result<(), Error>,
     ) -> Result<(), Error> {
-        let mut pool = Pool::default();
+        let pool = Pool::default();
 
-        let changelists = changelists.map(|cl| {
+        let changelists = options.changelists.map(|cl| {
             cl.iter()
                 .map(|cl| std::ffi::CString::new(*cl).unwrap())
                 .collect::<Vec<_>>()
@@ -496,13 +569,13 @@ impl Context {
             let receiver = Box::into_raw(Box::new(receiver));
             let err = svn_client_proplist4(
                 target.as_ptr() as *const i8,
-                &peg_revision.into(),
-                &revision.into(),
-                depth.into(),
+                &options.peg_revision.into(),
+                &options.revision.into(),
+                options.depth.into(),
                 changelists
                     .map(|cl| cl.as_ptr())
                     .unwrap_or(std::ptr::null()),
-                get_target_inherited_props.into(),
+                options.get_target_inherited_props.into(),
                 Some(wrap_proplist_receiver2),
                 receiver as *mut std::ffi::c_void,
                 &mut *self.0,
@@ -545,7 +618,7 @@ impl Context {
                 rps.as_ptr(),
                 Some(wrap_filter_callback),
                 filter_callback as *mut std::ffi::c_void,
-                Some(wrap_commit_callback2),
+                Some(crate::wrap_commit_callback2),
                 commit_callback as *mut std::ffi::c_void,
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -559,16 +632,10 @@ impl Context {
         &mut self,
         from_path_or_url: &str,
         to_path: impl AsCanonicalDirent<'a>,
-        peg_revision: Revision,
-        revision: Revision,
-        overwrite: bool,
-        ignore_externals: bool,
-        ignore_keywords: bool,
-        depth: Depth,
-        native_eol: crate::NativeEOL,
+        options: &ExportOptions,
     ) -> Result<Revnum, Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
-        let native_eol: Option<&str> = native_eol.into();
+        let native_eol: Option<&str> = options.native_eol.into();
         let native_eol = native_eol.map(|s| std::ffi::CString::new(s).unwrap());
         let mut revnum = 0;
         let to_path = to_path.as_canonical_dirent(std::rc::Rc::get_mut(&mut pool).unwrap());
@@ -577,31 +644,25 @@ impl Context {
                 &mut revnum,
                 from_path_or_url.as_ptr() as *const i8,
                 to_path.as_ptr(),
-                &peg_revision.into(),
-                &revision.into(),
-                overwrite as i32,
-                ignore_externals as i32,
-                ignore_keywords as i32,
-                depth as i32,
+                &options.peg_revision.into(),
+                &options.revision.into(),
+                options.overwrite as i32,
+                options.ignore_externals as i32,
+                options.ignore_keywords as i32,
+                options.depth as i32,
                 native_eol.map(|s| s.as_ptr()).unwrap_or(std::ptr::null()),
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
             );
             Error::from_raw(err)?;
-            Ok(revnum)
+            Ok(Revnum::from_raw(revnum).unwrap())
         }
     }
 
     pub fn commit(
         &mut self,
         targets: &[&str],
-        depth: Depth,
-        keep_locks: bool,
-        keep_changelists: bool,
-        commit_as_operations: bool,
-        include_file_externals: bool,
-        include_dir_externals: bool,
-        changelists: Option<&[&str]>,
+        options: &CommitOptions,
         revprop_table: std::collections::HashMap<&str, &str>,
         commit_callback: &dyn FnMut(&crate::CommitInfo) -> Result<(), Error>,
     ) -> Result<(), Error> {
@@ -618,7 +679,7 @@ impl Context {
                 ps.push(target.as_ptr() as *mut std::ffi::c_void);
             }
             let mut cl = apr::tables::ArrayHeader::in_pool(&pool, 0);
-            if let Some(changelists) = changelists {
+            if let Some(changelists) = options.changelists {
                 for changelist in changelists {
                     let changelist = std::ffi::CString::new(*changelist).unwrap();
                     cl.push(changelist.as_ptr() as *mut std::ffi::c_void);
@@ -627,15 +688,15 @@ impl Context {
             let commit_callback = Box::into_raw(Box::new(commit_callback));
             let err = svn_client_commit6(
                 ps.as_ptr(),
-                depth.into(),
-                keep_locks.into(),
-                keep_changelists.into(),
-                commit_as_operations.into(),
-                include_file_externals.into(),
-                include_dir_externals.into(),
+                options.depth.into(),
+                options.keep_locks.into(),
+                options.keep_changelists.into(),
+                options.commit_as_operations.into(),
+                options.include_file_externals.into(),
+                options.include_dir_externals.into(),
                 cl.as_ptr(),
                 rps.as_ptr(),
-                Some(wrap_commit_callback2),
+                Some(crate::wrap_commit_callback2),
                 commit_callback as *mut std::ffi::c_void,
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -648,20 +709,12 @@ impl Context {
     pub fn status(
         &mut self,
         path: &str,
-        revision: Revision,
-        depth: Depth,
-        get_all: bool,
-        check_out_of_date: bool,
-        check_working_copy: bool,
-        no_ignore: bool,
-        ignore_externals: bool,
-        depth_as_sticky: bool,
-        changelists: Option<&[&str]>,
+        options: &StatusOptions,
         status_func: &dyn FnMut(&'_ str, &'_ Status) -> Result<(), Error>,
     ) -> Result<Revnum, Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         let mut cl = apr::tables::ArrayHeader::in_pool(&pool, 0);
-        if let Some(changelists) = changelists {
+        if let Some(changelists) = options.changelists {
             for changelist in changelists {
                 let changelist = std::ffi::CString::new(*changelist).unwrap();
                 cl.push(changelist.as_ptr() as *mut std::ffi::c_void);
@@ -675,21 +728,21 @@ impl Context {
                 &mut revnum,
                 &mut *self.0,
                 path.as_ptr() as *const i8,
-                &revision.into(),
-                depth.into(),
-                get_all.into(),
-                check_out_of_date.into(),
-                check_working_copy.into(),
-                no_ignore.into(),
-                ignore_externals.into(),
-                depth_as_sticky.into(),
+                &options.revision.into(),
+                options.depth.into(),
+                options.get_all.into(),
+                options.check_out_of_date.into(),
+                options.check_working_copy.into(),
+                options.no_ignore.into(),
+                options.ignore_externals.into(),
+                options.depth_as_sticky.into(),
                 cl.as_ptr(),
                 Some(wrap_status_func),
                 status_func as *mut std::ffi::c_void,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
             );
             Error::from_raw(err)?;
-            Ok(revnum as Revnum)
+            Ok(Revnum::from_raw(revnum).unwrap())
         }
     }
 
@@ -732,7 +785,7 @@ impl Context {
                 strict_node_history.into(),
                 include_merged_revisions.into(),
                 rps.as_ptr(),
-                Some(wrap_log_entry_receiver),
+                Some(crate::wrap_log_entry_receiver),
                 log_entry_receiver as *mut std::ffi::c_void,
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -742,13 +795,67 @@ impl Context {
         }
     }
 
+    pub fn iter_logs(
+        &mut self,
+        targets: &[&str],
+        peg_revision: Revision,
+        revision_ranges: &[RevisionRange],
+        limit: i32,
+        discover_changed_paths: bool,
+        strict_node_history: bool,
+        include_merged_revisions: bool,
+        revprops: &[&str],
+    ) -> impl Iterator<Item = Result<LogEntry, Error>> {
+        // Create a channel between the worker and this thread
+        let (tx, rx) = std::sync::mpsc::channel();
+
+        let targets = targets.iter().map(|s| s.to_string()).collect::<Vec<_>>();
+        let revision_ranges = revision_ranges.to_vec();
+        let revprops = revprops.iter().map(|s| s.to_string()).collect::<Vec<_>>();
+        let mut client = self.clone();
+
+        std::thread::spawn(move || {
+            let r = client.log(
+                targets
+                    .iter()
+                    .map(|s| s.as_str())
+                    .collect::<Vec<_>>()
+                    .as_slice(),
+                peg_revision,
+                &revision_ranges,
+                limit,
+                discover_changed_paths,
+                strict_node_history,
+                include_merged_revisions,
+                revprops
+                    .iter()
+                    .map(|s| s.as_str())
+                    .collect::<Vec<_>>()
+                    .as_slice(),
+                &mut |log_entry| {
+                    tx.send(Ok(Some(log_entry.clone()))).unwrap();
+                    Ok(())
+                },
+            );
+            if let Err(e) = r {
+                tx.send(Err(e)).unwrap();
+            }
+            tx.send(Ok(None)).unwrap();
+        });
+
+        // Return an iterator that reads from the channel
+        rx.into_iter()
+            .take_while(|x| x.is_ok() && x.as_ref().unwrap().is_some())
+            .map(|x| x.transpose().unwrap())
+    }
+
     pub fn args_to_target_array(
         &mut self,
         mut os: apr::getopt::Getopt,
         known_targets: &[&str],
         keep_last_origpath_on_truepath_collision: bool,
     ) -> Result<Vec<String>, crate::Error> {
-        let mut pool = apr::pool::Pool::new();
+        let pool = apr::pool::Pool::new();
         let known_targets = known_targets
             .iter()
             .map(|s| std::ffi::CString::new(*s).unwrap())
@@ -779,25 +886,17 @@ impl Context {
             .collect::<Vec<_>>())
     }
 
-    pub fn vacuum(
-        &mut self,
-        path: &str,
-        remove_unversioned_items: bool,
-        remove_ignored_items: bool,
-        fix_recorded_timestamps: bool,
-        vacuum_pristines: bool,
-        include_externals: bool,
-    ) -> Result<(), Error> {
+    pub fn vacuum(&mut self, path: &str, options: &VacuumOptions) -> Result<(), Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         let path = std::ffi::CString::new(path).unwrap();
         unsafe {
             let err = svn_client_vacuum(
-                path.as_ptr() as *const i8,
-                remove_unversioned_items.into(),
-                remove_ignored_items.into(),
-                fix_recorded_timestamps.into(),
-                vacuum_pristines.into(),
-                include_externals.into(),
+                path.as_ptr(),
+                options.remove_unversioned_items.into(),
+                options.remove_ignored_items.into(),
+                options.fix_recorded_timestamps.into(),
+                options.vacuum_pristines.into(),
+                options.include_externals.into(),
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
             );
@@ -806,25 +905,17 @@ impl Context {
         }
     }
 
-    pub fn cleanup(
-        &mut self,
-        path: &str,
-        break_locks: bool,
-        fix_recorded_timestamps: bool,
-        clear_dav_cache: bool,
-        vacuum_pristines: bool,
-        include_externals: bool,
-    ) -> Result<(), Error> {
+    pub fn cleanup(&mut self, path: &str, options: &CleanupOptions) -> Result<(), Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         let path = std::ffi::CString::new(path).unwrap();
         unsafe {
             let err = svn_client_cleanup2(
-                path.as_ptr() as *const i8,
-                break_locks.into(),
-                fix_recorded_timestamps.into(),
-                clear_dav_cache.into(),
-                vacuum_pristines.into(),
-                include_externals.into(),
+                path.as_ptr(),
+                options.break_locks.into(),
+                options.fix_recorded_timestamps.into(),
+                options.clear_dav_cache.into(),
+                options.vacuum_pristines.into(),
+                options.include_externals.into(),
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
             );
@@ -847,8 +938,8 @@ impl Context {
         unsafe {
             let err = svn_client_relocate2(
                 path.as_ptr(),
-                from.as_ptr() as *const i8,
-                to.as_ptr() as *const i8,
+                from.as_ptr(),
+                to.as_ptr(),
                 ignore_externals.into(),
                 &mut *self.0,
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
@@ -883,9 +974,7 @@ impl Context {
         &mut self,
         path_or_url: &str,
         stream: &mut dyn std::io::Write,
-        peg_revision: Revision,
-        revision: Revision,
-        expand_keywords: bool,
+        options: &CatOptions,
     ) -> Result<HashMap<String, Vec<u8>>, Error> {
         let mut pool = std::rc::Rc::new(Pool::default());
         let path_or_url = std::ffi::CString::new(path_or_url).unwrap();
@@ -896,9 +985,9 @@ impl Context {
                 &mut props,
                 s.as_mut_ptr(),
                 path_or_url.as_ptr(),
-                &peg_revision.into(),
-                &revision.into(),
-                expand_keywords.into(),
+                &options.peg_revision.into(),
+                &options.revision.into(),
+                options.expand_keywords.into(),
                 self.as_mut_ptr(),
                 std::rc::Rc::get_mut(&mut pool).unwrap().as_mut_ptr(),
                 apr::pool::Pool::new().as_mut_ptr(),
@@ -998,7 +1087,10 @@ impl Context {
                 scratch_pool.as_mut_ptr(),
             );
             Error::from_raw(err)?;
-            Ok((min_revision, max_revision))
+            Ok((
+                Revnum::from_raw(min_revision).unwrap(),
+                Revnum::from_raw(max_revision).unwrap(),
+            ))
         }
     }
 
@@ -1020,7 +1112,7 @@ impl Context {
     }
 
     pub fn get_repos_root(&mut self, path_or_url: &str) -> Result<(String, String), Error> {
-        let mut pool = Pool::default();
+        let pool = Pool::default();
         let path_or_url = std::ffi::CString::new(path_or_url).unwrap();
         let mut repos_root: *const i8 = std::ptr::null();
         let mut repos_uuid: *const i8 = std::ptr::null();
@@ -1028,7 +1120,7 @@ impl Context {
             let err = crate::generated::svn_client_get_repos_root(
                 &mut repos_root,
                 &mut repos_uuid,
-                path_or_url.as_ptr() as *const i8,
+                path_or_url.as_ptr(),
                 self.as_mut_ptr(),
                 pool.as_mut_ptr(),
                 pool.as_mut_ptr(),
@@ -1047,6 +1139,7 @@ impl Context {
         }
     }
 
+    #[cfg(feature = "ra")]
     pub fn open_raw_session(
         &mut self,
         url: &str,
@@ -1055,12 +1148,12 @@ impl Context {
         let url = std::ffi::CString::new(url).unwrap();
         let wri_path = std::ffi::CString::new(wri_path.to_str().unwrap()).unwrap();
         let session = PooledPtr::initialize(|pool| unsafe {
-            let mut scratch_pool = Pool::default();
+            let scratch_pool = Pool::default();
             let mut session: *mut crate::generated::svn_ra_session_t = std::ptr::null_mut();
             let err = crate::generated::svn_client_open_ra_session2(
                 &mut session,
-                url.as_ptr() as *const i8,
-                wri_path.as_ptr() as *const i8,
+                url.as_ptr(),
+                wri_path.as_ptr(),
                 self.as_mut_ptr(),
                 pool.as_mut_ptr(),
                 scratch_pool.as_mut_ptr(),
@@ -1083,7 +1176,7 @@ impl Context {
         changelists: Option<&[&str]>,
         receiver: &dyn FnMut(&Info) -> Result<(), Error>,
     ) -> Result<(), Error> {
-        let mut pool = Pool::default();
+        let pool = Pool::default();
         let abspath_or_url = std::ffi::CString::new(abspath_or_url).unwrap();
         let changelists = changelists.map(|cl| {
             cl.iter()
@@ -1092,14 +1185,14 @@ impl Context {
         });
         let changelists = changelists.as_ref().map(|cl| {
             cl.iter()
-                .map(|cl| cl.as_ptr() as *const i8)
+                .map(|cl| cl.as_ptr())
                 .collect::<apr::tables::ArrayHeader<_>>()
         });
         let mut receiver = receiver;
         let receiver = &mut receiver as *mut _ as *mut std::ffi::c_void;
         unsafe {
             let err = crate::generated::svn_client_info4(
-                abspath_or_url.as_ptr() as *const i8,
+                abspath_or_url.as_ptr(),
                 &peg_revision.into(),
                 &revision.into(),
                 depth.into(),
@@ -1124,6 +1217,217 @@ impl Default for Context {
     }
 }
 
+pub struct Status(pub(crate) *const crate::generated::svn_client_status_t);
+
+impl Status {
+    pub fn kind(&self) -> crate::NodeKind {
+        unsafe { (*self.0).kind.into() }
+    }
+
+    pub fn local_abspath(&self) -> &str {
+        unsafe {
+            std::ffi::CStr::from_ptr((*self.0).local_abspath)
+                .to_str()
+                .unwrap()
+        }
+    }
+
+    pub fn filesize(&self) -> i64 {
+        unsafe { (*self.0).filesize }
+    }
+
+    pub fn versioned(&self) -> bool {
+        unsafe { (*self.0).versioned != 0 }
+    }
+
+    pub fn conflicted(&self) -> bool {
+        unsafe { (*self.0).conflicted != 0 }
+    }
+
+    pub fn node_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).node_status.into() }
+    }
+
+    pub fn text_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).text_status.into() }
+    }
+
+    pub fn prop_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).prop_status.into() }
+    }
+
+    pub fn wc_is_locked(&self) -> bool {
+        unsafe { (*self.0).wc_is_locked != 0 }
+    }
+
+    pub fn copied(&self) -> bool {
+        unsafe { (*self.0).copied != 0 }
+    }
+
+    pub fn repos_root_url(&self) -> &str {
+        unsafe {
+            std::ffi::CStr::from_ptr((*self.0).repos_root_url)
+                .to_str()
+                .unwrap()
+        }
+    }
+
+    pub fn repos_uuid(&self) -> &str {
+        unsafe {
+            std::ffi::CStr::from_ptr((*self.0).repos_uuid)
+                .to_str()
+                .unwrap()
+        }
+    }
+
+    pub fn repos_relpath(&self) -> &str {
+        unsafe {
+            std::ffi::CStr::from_ptr((*self.0).repos_relpath)
+                .to_str()
+                .unwrap()
+        }
+    }
+
+    pub fn revision(&self) -> Revnum {
+        Revnum::from_raw(unsafe { (*self.0).revision }).unwrap()
+    }
+
+    pub fn changed_rev(&self) -> Revnum {
+        Revnum::from_raw(unsafe { (*self.0).changed_rev }).unwrap()
+    }
+
+    pub fn changed_date(&self) -> apr::time::Time {
+        unsafe { apr::time::Time::from((*self.0).changed_date) }
+    }
+
+    pub fn changed_author(&self) -> &str {
+        unsafe {
+            std::ffi::CStr::from_ptr((*self.0).changed_author)
+                .to_str()
+                .unwrap()
+        }
+    }
+
+    pub fn switched(&self) -> bool {
+        unsafe { (*self.0).switched != 0 }
+    }
+
+    pub fn file_external(&self) -> bool {
+        unsafe { (*self.0).file_external != 0 }
+    }
+
+    pub fn lock(&self) -> Option<&crate::Lock> {
+        todo!()
+    }
+
+    pub fn changelist(&self) -> Option<&str> {
+        unsafe {
+            if (*self.0).changelist.is_null() {
+                None
+            } else {
+                Some(
+                    std::ffi::CStr::from_ptr((*self.0).changelist)
+                        .to_str()
+                        .unwrap(),
+                )
+            }
+        }
+    }
+
+    pub fn depth(&self) -> crate::Depth {
+        unsafe { (*self.0).depth.into() }
+    }
+
+    pub fn ood_kind(&self) -> crate::NodeKind {
+        unsafe { (*self.0).ood_kind.into() }
+    }
+
+    pub fn repos_node_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).repos_node_status.into() }
+    }
+
+    pub fn repos_text_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).repos_text_status.into() }
+    }
+
+    pub fn repos_prop_status(&self) -> crate::StatusKind {
+        unsafe { (*self.0).repos_prop_status.into() }
+    }
+
+    pub fn repos_lock(&self) -> Option<crate::Lock> {
+        todo!()
+    }
+
+    pub fn ood_changed_rev(&self) -> Option<Revnum> {
+        Revnum::from_raw(unsafe { (*self.0).ood_changed_rev })
+    }
+
+    pub fn ood_changed_author(&self) -> Option<&str> {
+        unsafe {
+            if (*self.0).ood_changed_author.is_null() {
+                None
+            } else {
+                Some(
+                    std::ffi::CStr::from_ptr((*self.0).ood_changed_author)
+                        .to_str()
+                        .unwrap(),
+                )
+            }
+        }
+    }
+
+    pub fn moved_from_abspath(&self) -> Option<&str> {
+        unsafe {
+            if (*self.0).moved_from_abspath.is_null() {
+                None
+            } else {
+                Some(
+                    std::ffi::CStr::from_ptr((*self.0).moved_from_abspath)
+                        .to_str()
+                        .unwrap(),
+                )
+            }
+        }
+    }
+
+    pub fn moved_to_abspath(&self) -> Option<&str> {
+        unsafe {
+            if (*self.0).moved_to_abspath.is_null() {
+                None
+            } else {
+                Some(
+                    std::ffi::CStr::from_ptr((*self.0).moved_to_abspath)
+                        .to_str()
+                        .unwrap(),
+                )
+            }
+        }
+    }
+}
+
+pub struct Conflict(pub(crate) apr::pool::PooledPtr<crate::generated::svn_client_conflict_t>);
+
+impl Conflict {
+    pub fn prop_get_description(&mut self) -> Result<String, Error> {
+        let pool = apr::pool::Pool::new();
+        let mut description: *const i8 = std::ptr::null_mut();
+        let err = unsafe {
+            crate::generated::svn_client_conflict_prop_get_description(
+                &mut description,
+                self.0.as_mut_ptr(),
+                pool.as_mut_ptr(),
+                pool.as_mut_ptr(),
+            )
+        };
+
+        Error::from_raw(err)?;
+        Ok(unsafe { std::ffi::CStr::from_ptr(description) }
+            .to_str()
+            .unwrap()
+            .to_owned())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1147,38 +1451,15 @@ mod tests {
             .checkout(
                 url,
                 &td.path().join("wc"),
-                Revision::Head,
-                Revision::Head,
-                Depth::Infinity,
-                false,
-                false,
+                &CheckoutOptions {
+                    peg_revision: Revision::Head,
+                    revision: Revision::Head,
+                    depth: Depth::Infinity,
+                    ignore_externals: false,
+                    allow_unver_obstructions: false,
+                },
             )
             .unwrap();
-        assert_eq!(revnum, 0);
-    }
-}
-
-pub struct Status(pub(crate) *const crate::generated::svn_client_status_t);
-
-pub struct Conflict(pub(crate) apr::pool::PooledPtr<crate::generated::svn_client_conflict_t>);
-
-impl Conflict {
-    pub fn prop_get_description(&mut self) -> Result<String, Error> {
-        let mut pool = apr::pool::Pool::new();
-        let mut description: *const i8 = std::ptr::null_mut();
-        let err = unsafe {
-            crate::generated::svn_client_conflict_prop_get_description(
-                &mut description,
-                self.0.as_mut_ptr(),
-                pool.as_mut_ptr(),
-                pool.as_mut_ptr(),
-            )
-        };
-
-        Error::from_raw(err)?;
-        Ok(unsafe { std::ffi::CStr::from_ptr(description) }
-            .to_str()
-            .unwrap()
-            .to_owned())
+        assert_eq!(revnum, Revnum(0));
     }
 }
